package com.s5gi.giGuns.guns;

import com.s5gi.giGuns.giGuns;
import org.bukkit.Color;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Sound;
import org.bukkit.block.Block;
import org.bukkit.entity.LivingEntity;
import org.bukkit.inventory.CraftingRecipe;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.components.CustomModelDataComponent;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public interface IGun {
    String name();
    List<String> description();
    Material getItem();
    BulletType bulletType();
    List<Sound> sound();
    float soundPitch();
    void onHitPlayer(Location location, LivingEntity entity);
    void onHitBlock(Location location, Block block);
    double bulletSpeed();
    double damage();
    //in ticks
    int cooldown();
    default int textureId() {
        return 0;
    }
    CraftingRecipe recipe();
    default List<String> getLore() {
        List<String> lore = new ArrayList<>();
        lore.add(giGuns.color("&7Ammo Type: " + this.bulletType().prettyName()));
        if (!this.description().isEmpty()) {
            lore.add("");
            this.description().forEach(s -> lore.add(giGuns.color(s)));
        }
        return lore;
    }
    default ItemStack compileGun() {
        IGun gun = this;
        ItemStack gunItem = new ItemStack(gun.getItem());
        ItemMeta meta = gunItem.getItemMeta();
        meta.setItemName(giGuns.color(gun.name()));
        meta.setLore(gun.getLore());
        CustomModelDataComponent component = meta.getCustomModelDataComponent();
        component.setStrings(List.of("gunmodel" + gun.textureId()));
        meta.setCustomModelDataComponent(component);
        gunItem.setItemMeta(meta);
        return gunItem;
    }
    default boolean gravity() {return false;}
    default IBullet compileBullet() {
        IGun gun = this;
        return new IBullet() {
            @Override
            public Material mat() {
                return Material.POLISHED_BLACKSTONE_BUTTON;
            }

            @Override
            public IGun parentGun() {
                return gun;
            }
        };
    }
}
